import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import {
  CommandBus,
  CommandHandler,
  ICommandHandler,
  QueryBus,
} from '@nestjs/cqrs';
import { FindAdsByUserIdQuery } from '../../queries/find-ads-by-user-id/find-ads-by-user-id.query';
import { DeleteAdCommand } from '../delete-ad/delete-ad.command';
import { DeleteUserAdsCommand } from './delete-user-ads.command';

@CommandHandler(DeleteUserAdsCommand)
export class DeleteUserAdsService implements ICommandHandler {
  constructor(
    private readonly queryBus: QueryBus,
    private readonly commandBus: CommandBus,
  ) {}

  async execute(command: DeleteUserAdsCommand): Promise<void> {
    const ads: AdEntity[] = await this.queryBus.execute(
      new FindAdsByUserIdQuery(command.id),
    );
    await Promise.all(
      ads.map((ad) =>
        this.commandBus.execute(new DeleteAdCommand({ id: ad.id })),
      ),
    );
  }
}
