import { Injectable } from '@nestjs/common';
import { RabbitSubscribe } from '@mobicoop/message-broker-module';
import { CommandBus } from '@nestjs/cqrs';
import { MATCHER_AD_CREATION_FAILED_MESSAGE_HANDLER } from '@src/app.constants';
import { MatcherAdCreationFailedIntegrationEvent } from './matcher-ad.types';
import { InvalidateAdCommand } from '@modules/ad/core/application/commands/invalidate-ad/invalidate-ad.command';

@Injectable()
export class MatcherAdCreationFailedMessageHandler {
  constructor(private readonly commandBus: CommandBus) {}

  @RabbitSubscribe({
    name: MATCHER_AD_CREATION_FAILED_MESSAGE_HANDLER,
  })
  public async matcherAdCreationFailed(message: string) {
    try {
      const matcherAdCreationFailedIntegrationEvent: MatcherAdCreationFailedIntegrationEvent =
        JSON.parse(message);
      await this.commandBus.execute(
        new InvalidateAdCommand({
          id: matcherAdCreationFailedIntegrationEvent.id,
        }),
      );
    } catch (error: any) {
      // do not throw error to acknowledge incoming message
      // error handling should be done in the command handler, if relevant
    }
  }
}
