import { IntegrationEvent } from '@mobicoop/ddd-library';
import { RabbitSubscribe } from '@mobicoop/message-broker-module';
import { DeleteUserAdsCommand } from '@modules/ad/core/application/commands/delete-user-ads/delete-user-ads.command';
import { Injectable } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { USER_DELETED_MESSAGE_HANDLER } from '@src/app.constants';

type UserDeletedEvent = IntegrationEvent;

@Injectable()
export class UserDeletedMessageHandler {
  constructor(private readonly commandBus: CommandBus) {}

  @RabbitSubscribe({
    name: USER_DELETED_MESSAGE_HANDLER,
  })
  public async userDeleted(message: string) {
    const deletedUser: UserDeletedEvent = JSON.parse(message);
    await this.commandBus.execute(
      new DeleteUserAdsCommand({ id: deletedUser.id }),
    );
  }
}
