import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { FindAdByIdQuery } from '@modules/ad/core/application/queries/find-ad-by-id/find-ad-by-id.query';
import { FindAdByIdQueryHandler } from '@modules/ad/core/application/queries/find-ad-by-id/find-ad-by-id.query-handler';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { Test, TestingModule } from '@nestjs/testing';
import { punctualPassengerCreateAdProps } from './ad.fixtures';

const ad: AdEntity = AdEntity.create(punctualPassengerCreateAdProps());

const mockAdRepository = {
  findOneById: jest.fn().mockImplementation(() => ad),
};

describe('find-ad-by-id.query-handler', () => {
  let findAdByIdQueryHandler: FindAdByIdQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        FindAdByIdQueryHandler,
      ],
    }).compile();

    findAdByIdQueryHandler = module.get<FindAdByIdQueryHandler>(
      FindAdByIdQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(findAdByIdQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return an ad', async () => {
      const findAdbyIdQuery = new FindAdByIdQuery(
        'dd264806-13b4-4226-9b18-87adf0ad5dd1',
      );
      const ad: AdEntity =
        await findAdByIdQueryHandler.execute(findAdbyIdQuery);
      expect(ad.getProps().fromDate).toBe('2023-06-22');
    });
  });
});
