import { AggregateID } from '@mobicoop/ddd-library';
import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { ValidateAdCommand } from '@modules/ad/core/application/commands/validate-ad/validate-ad.command';
import { ValidateAdService } from '@modules/ad/core/application/commands/validate-ad/validate-ad.service';
import { AdEntity } from '@modules/ad/core/domain/ad.entity';
import { CreateAdProps, Frequency } from '@modules/ad/core/domain/ad.types';
import { WaypointProps } from '@modules/ad/core/domain/value-objects/waypoint.value-object';
import { Test, TestingModule } from '@nestjs/testing';

const originWaypointProps: WaypointProps = {
  position: 0,
  address: {
    houseNumber: '5',
    street: 'Avenue Foch',
    locality: 'Nancy',
    postalCode: '54000',
    country: 'France',
    coordinates: {
      lat: 48.689445,
      lon: 6.17651,
    },
  },
};
const destinationWaypointProps: WaypointProps = {
  position: 1,
  address: {
    locality: 'Paris',
    postalCode: '75000',
    country: 'France',
    coordinates: {
      lat: 48.8566,
      lon: 2.3522,
    },
  },
};
const baseCreateAdProps = {
  userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
  waypoints: [originWaypointProps, destinationWaypointProps],
};
const punctualCreateAdProps = {
  fromDate: '2023-06-22',
  toDate: '2023-06-22',
  schedule: [
    {
      day: 4,
      time: '08:30',
      margin: 900,
    },
  ],
  frequency: Frequency.PUNCTUAL,
};
const punctualPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  driver: false,
  passenger: true,
};

const ad: AdEntity = AdEntity.create(punctualPassengerCreateAdProps);

const mockAdRepository = {
  findOneById: jest.fn().mockImplementation(() => ad),
  update: jest.fn().mockImplementation(() => ad.id),
};

describe('Validate Ad Service', () => {
  let validateAdService: ValidateAdService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        ValidateAdService,
      ],
    }).compile();

    validateAdService = module.get<ValidateAdService>(ValidateAdService);
  });

  it('should be defined', () => {
    expect(validateAdService).toBeDefined();
  });

  describe('execution', () => {
    it('should validate an ad', async () => {
      jest.spyOn(ad, 'valid');
      const validateAdCommand = new ValidateAdCommand(ad.id);
      const result: AggregateID =
        await validateAdService.execute(validateAdCommand);
      expect(result).toBe(ad.id);
      expect(ad.valid).toHaveBeenCalledTimes(1);
    });
  });
});
